//
//  SlideSaver.m
//  SlideSaver
//
//  Created by Stphane Sudre on Fri Oct 20 2000.
//  Copyright (c) 2000 Stphane Sudre. All rights reserved.
//

#import <AppKit/AppKit.h>
#import "SlideSaver.h"

@implementation SlideSaver

- (id)initWithFrame:(NSRect)frameRect isPreview:(BOOL) preview
{
    ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:@"fr.whitebox.slideSaver"];
    
    if (![super initWithFrame:frameRect isPreview:preview]) return nil;
    
    imageRect_=previousImageRect_=frameRect;
    
    directoryLocation_=[defaults stringForKey:@"ImageDirectory"];
    
    if (directoryLocation_==nil)
    {
        directoryLocation_=[[NSString alloc]initWithString:NSHomeDirectory()];
        refreshPeriod_=4;
        
        [defaults setObject:directoryLocation_ forKey:@"ImageDirectory"];
        [defaults setInteger:refreshPeriod_ forKey:@"RefreshPeriod"];
    
        [defaults synchronize];
    }
    else
    {
        refreshPeriod_=[defaults integerForKey:@"RefreshPeriod"];
    }
    
    return self;
}

- (void)drawRect:(NSRect)rects
{
    [[NSColor blackColor] set];
    NSRectFill(rects);
}

- (void)setFrameSize:(NSSize)newSize
{
    [super setFrameSize:newSize];
    
    imageRect_=previousImageRect_=[self frame];
}

- (void)oneStep
{
    float imageWidth,imageHeight;
    NSRect frame;
    NSString *file;
    
    file = [enumerator_ nextObject];
    
    if (file==nil)
    {
        // The End, so let's start it again !
        
        [enumerator_ release];
        
        enumerator_= [[[NSFileManager defaultManager] enumeratorAtPath:directoryLocation_] retain];
        
        file = [enumerator_ nextObject];
    }
    
    if (file!=nil)
    {
    	imageRep_=[[NSBitmapImageRep  imageRepWithData:[NSData dataWithContentsOfFile:file]] retain];
    
        if (imageRep_!=nil)
        {
            imageWidth=[imageRep_ pixelsWide];
            imageHeight=[imageRep_ pixelsHigh];
        
            if (imageWidth>imageRect_.size.width || imageHeight>imageRect_.size.height)
            {
                // We need to scale the image
                
                float h_ratio,v_ratio;
                
                h_ratio=imageRect_.size.width/imageWidth;
                v_ratio=imageRect_.size.height/imageHeight;
                
                if (h_ratio>v_ratio) h_ratio=v_ratio;
                
                imageWidth*=h_ratio;
                imageHeight*=h_ratio;
            }
            
            // Center the image
            
            frame.origin.x=imageRect_.origin.x+(imageRect_.size.width-imageWidth)/2;
            frame.origin.y=imageRect_.origin.y+(imageRect_.size.height -imageHeight)/2;
        
            frame.size.width=imageWidth;
            frame.size.height=imageHeight;
            
            [[NSColor blackColor] set];
            
            previousImageRect_= NSInsetRect(previousImageRect_, -1.0, -1.0);
            [NSBezierPath fillRect:previousImageRect_];
            
            [imageRep_ drawInRect:frame];
            
            previousImageRect_=frame;
            
            [imageRep_ release];
        }
    }
}

- (void)startAnimation
{
    
    [ [NSFileManager defaultManager] changeCurrentDirectoryPath:directoryLocation_];
    
    enumerator_= [[[NSFileManager defaultManager] enumeratorAtPath:directoryLocation_] retain];
    
    [super startAnimation];
}

- (void)stopAnimation
{
    [super stopAnimation];
}


- (double)animationTimeInterval
{
    return refreshPeriod_;
}

- (BOOL) hasConfigureSheet
{
    return YES;
}

- (NSWindow*)configureSheet
{
    if (configureSheet_ == nil)
    {
        [NSBundle loadNibNamed:@"ConfigureSheet" owner:self];
        
        if (directoryLocation_!=nil)
        {
            [directoryTextView_ setStringValue:directoryLocation_];
        }
        
        [periodView_ setIntValue:refreshPeriod_];
    }
    return configureSheet_;
}

- (IBAction)closeSheet:(id)sender
{
    ScreenSaverDefaults *defaults = [ScreenSaverDefaults defaultsForModuleWithName:@"fr.whitebox.slideSaver"];

    [self setPeriod:nil];
    
    [defaults setObject:directoryLocation_ forKey:@"ImageDirectory"];
    [defaults setInteger:refreshPeriod_ forKey:@"RefreshPeriod"];
    
    [defaults synchronize];
    
    [NSApp endSheet:configureSheet_];
}

- (void) selectDirectory:(id) sender
{
    NSOpenPanel *oPanel = [NSOpenPanel openPanel];
    int result;
    
    [oPanel setAllowsMultipleSelection:NO];
    [oPanel setCanChooseDirectories:YES];
    [oPanel setCanChooseFiles:NO];
    
    
    result = [oPanel runModalForDirectory:directoryLocation_ file:nil types:nil];
    if (result == NSOKButton)
    {
        NSArray *filesToOpen = [oPanel filenames];
        NSString *aFile = [filesToOpen objectAtIndex:0];
        
        if (directoryLocation_!=nil)
        {
            [directoryLocation_ release];
        }
        directoryLocation_=[[NSString alloc] initWithString:aFile];
        
        [directoryTextView_ setStringValue:directoryLocation_];
    }
}

- (void) setPeriod:(id) sender
{
    int tRefreshPeriod;
    
    tRefreshPeriod=[periodView_ intValue];
    if (tRefreshPeriod<1)
    {
        [periodView_ setIntValue:refreshPeriod_];
    }
    else
    {
        [self setAnimationTimeInterval:(refreshPeriod_=tRefreshPeriod)];
    }
}

@end
